/* Project: Paddle Puck
 * Author: Dennis Busch, http://www.dennisbusch.de, Dennis[minus]Busch[at]gmx[dot]net
 */

package paddlepuckclient;

import paddlepuckGUI.GUIComponentHandlers;
import util.Logging;
import paddlepuckprotocol.PaddlePuckClient;

/**
 * The front-end for the Paddle Puck Client.
 */
public class PaddlePuckClientGUI extends java.awt.Frame
{  
  private PaddlePuckClientView view = null;
  private PaddlePuckClient client = null;
  private Logging logging = null;
  
  /** Creates new form PaddlePuckClientGUI 
   * @param antialias set to true to enable anti alias in renderer
   */
  public PaddlePuckClientGUI(boolean antialias)
  {
    initComponents();
    
    // init title and logging
    this.setTitle(String.format("Paddle Puck Client (%1$d)", PaddlePuckClient.Version));
    this.logging = new Logging(this.listLog);
    
    try
    {
      // init network client
      client = new PaddlePuckClient(this.logging);
    
      // init view
      view = new PaddlePuckClientView(this.logging, this, this.client, panelView.getWidth(), panelView.getHeight(), antialias);
      panelView.add(view);
      
      // tell client about the view (will get signalled for updates)
      client.setView(view);
      
      // start client
      client.StartRegularUpdatesThread();
    }
    catch(Exception e)
    {
      this.logging.LogException(e);
    }
  }
  
  /**
   * connects the client object and updates the GUI state
   */
  public void Connect()
  {
    try
    {
      int port = Integer.parseInt(this.textFieldPort.getText());
      int localPort = Integer.parseInt(this.textFieldLocalPort.getText());
      
      setGUIstate(true);
      this.logging.LogMessage("trying to connect...");
      this.client.ConnectToServer(this.textFieldHost.getText(), port, localPort);
      
      this.view.SetCameraToPlayer(this.client.getPlayerNumber());
    }
    catch (Exception e)
    {
      this.logging.LogException(e);
      this.logging.LogMessage("Failed to connect. Check cable/port/server/firewall/etc...");
      
      setGUIstate(false);
    }
  }
  
  /**
   * disconnects the client object and updates the GUI state
   */
  public void Disconnect()
  {
    this.client.DisconnectFromServer();
    setGUIstate(false);
    this.view.SetCameraToPlayer(0);
  }

  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        listLog = new java.awt.List();
        labelHost = new java.awt.Label();
        textFieldHost = new java.awt.TextField();
        labelPort = new java.awt.Label();
        textFieldPort = new java.awt.TextField();
        buttonConnect = new java.awt.Button();
        buttonDisconnect = new java.awt.Button();
        panelView = new java.awt.Panel();
        label1 = new java.awt.Label();
        lblLocalPort = new java.awt.Label();
        textFieldLocalPort = new java.awt.TextField();

        setMinimumSize(new java.awt.Dimension(640, 480));
        setResizable(false);
        addWindowFocusListener(new java.awt.event.WindowFocusListener() {
            public void windowGainedFocus(java.awt.event.WindowEvent evt) {
            }
            public void windowLostFocus(java.awt.event.WindowEvent evt) {
                formWindowLostFocus(evt);
            }
        });
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        listLog.setName("listLog"); // NOI18N

        labelHost.setName("labelHost"); // NOI18N
        labelHost.setText("Host:");

        textFieldHost.setName("textFieldHost"); // NOI18N
        textFieldHost.setText("127.0.0.1");

        labelPort.setName("labelPort"); // NOI18N
        labelPort.setText("Host-Port:");

        textFieldPort.setName("textFieldPort"); // NOI18N
        textFieldPort.setText("42000");
        textFieldPort.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                textFieldPortKeyTyped(evt);
            }
        });

        buttonConnect.setLabel("Connect");
        buttonConnect.setName("buttonConnect"); // NOI18N
        buttonConnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonConnectActionPerformed(evt);
            }
        });

        buttonDisconnect.setEnabled(false);
        buttonDisconnect.setLabel("Disconnect");
        buttonDisconnect.setName("buttonDisconnect"); // NOI18N
        buttonDisconnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonDisconnectActionPerformed(evt);
            }
        });

        panelView.setName("panelView"); // NOI18N

        javax.swing.GroupLayout panelViewLayout = new javax.swing.GroupLayout(panelView);
        panelView.setLayout(panelViewLayout);
        panelViewLayout.setHorizontalGroup(
            panelViewLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 620, Short.MAX_VALUE)
        );
        panelViewLayout.setVerticalGroup(
            panelViewLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 310, Short.MAX_VALUE)
        );

        label1.setAlignment(java.awt.Label.CENTER);
        label1.setText("click anywhere in the view below to grab the paddle / move mouse to move paddle / click again to release paddle");

        lblLocalPort.setName("lblLocalPort"); // NOI18N
        lblLocalPort.setText("Local-Port:");
        lblLocalPort.setVisible(false);

        textFieldLocalPort.setName("tbLocalPort"); // NOI18N
        textFieldLocalPort.setText("42001");
        textFieldLocalPort.setVisible(false);
        textFieldLocalPort.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                textFieldLocalPortKeyTyped(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(listLog, javax.swing.GroupLayout.DEFAULT_SIZE, 620, Short.MAX_VALUE)
                    .addComponent(panelView, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(label1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 620, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                        .addComponent(labelHost, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(textFieldHost, javax.swing.GroupLayout.PREFERRED_SIZE, 115, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(labelPort, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(textFieldPort, javax.swing.GroupLayout.PREFERRED_SIZE, 62, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(lblLocalPort, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(textFieldLocalPort, javax.swing.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE)
                        .addGap(26, 26, 26)
                        .addComponent(buttonConnect, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(buttonDisconnect, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(label1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(panelView, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(buttonConnect, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(buttonDisconnect, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                        .addComponent(textFieldHost, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(labelHost, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(labelPort, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(textFieldPort, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(lblLocalPort, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(textFieldLocalPort, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(listLog, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

  /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
      this.client.StopRegularUpdatesThread();
      this.client.DisconnectFromServer();
      System.exit(0);
    }//GEN-LAST:event_exitForm

  private void textFieldPortKeyTyped(java.awt.event.KeyEvent evt)//GEN-FIRST:event_textFieldPortKeyTyped
  {//GEN-HEADEREND:event_textFieldPortKeyTyped
    GUIComponentHandlers.PortTextFieldKeyTyped(evt);
  }//GEN-LAST:event_textFieldPortKeyTyped

  private void buttonConnectActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_buttonConnectActionPerformed
  {//GEN-HEADEREND:event_buttonConnectActionPerformed
    this.Connect();
  }//GEN-LAST:event_buttonConnectActionPerformed

  private void buttonDisconnectActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_buttonDisconnectActionPerformed
  {//GEN-HEADEREND:event_buttonDisconnectActionPerformed
    this.Disconnect();
  }//GEN-LAST:event_buttonDisconnectActionPerformed

  private void formWindowLostFocus(java.awt.event.WindowEvent evt)//GEN-FIRST:event_formWindowLostFocus
  {//GEN-HEADEREND:event_formWindowLostFocus
    this.view.setMouseCaptured(false);
  }//GEN-LAST:event_formWindowLostFocus

  private void textFieldLocalPortKeyTyped(java.awt.event.KeyEvent evt)//GEN-FIRST:event_textFieldLocalPortKeyTyped
  {//GEN-HEADEREND:event_textFieldLocalPortKeyTyped
    GUIComponentHandlers.PortTextFieldKeyTyped(evt);
  }//GEN-LAST:event_textFieldLocalPortKeyTyped

  /**
   * used to enable/disable GUI elements according to connection state
   * @param connected 
   */
  private void setGUIstate(boolean connected)
  {
    if(connected)
    {
      textFieldPort.setEnabled(false);
      textFieldLocalPort.setEnabled(false);
      textFieldHost.setEnabled(false);
      buttonConnect.setEnabled(false);
      buttonDisconnect.setEnabled(true);
    }
    else
    {
      textFieldPort.setEnabled(true);
      textFieldLocalPort.setEnabled(true);
      textFieldHost.setEnabled(true);
      buttonConnect.setEnabled(true);
      buttonDisconnect.setEnabled(false);
    }
  }

  private static boolean aa = false; // for anti alias commandline option
  /**
   * @param args the command line arguments
   */
  public static void main(String args[])
  {
    for(int i=0; i < args.length; i++) //
    {
      if(args[i].compareTo("/aa") == 0) // enable anti alias?
      {
        aa = true;
      }
    }
    
    java.awt.EventQueue.invokeLater(new Runnable()
    {
      @Override
      public void run()
      {
        new PaddlePuckClientGUI(aa).setVisible(true);
      }
    });
  }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private java.awt.Button buttonConnect;
    private java.awt.Button buttonDisconnect;
    private java.awt.Label label1;
    private java.awt.Label labelHost;
    private java.awt.Label labelPort;
    private java.awt.Label lblLocalPort;
    private java.awt.List listLog;
    private java.awt.Panel panelView;
    private java.awt.TextField textFieldHost;
    private java.awt.TextField textFieldLocalPort;
    private java.awt.TextField textFieldPort;
    // End of variables declaration//GEN-END:variables
}
