/* Project: Paddle Puck
 * Author: Dennis Busch, http://www.dennisbusch.de, Dennis[minus]Busch[at]gmx[dot]net
 */

package paddlepuckprotocol;

/**
 * This enumeration contains the messages used in PaddlePuck Client/Server communication.
 * Each message starts with an integer (which is the ordinal corresponding to the enum value).
 * 
 * These messages can be received by either side at any time after the initial connection
 * protocol (version check/player info) has been performed.
 */
public enum PaddlePuckMessage
{
  Undefined, // (not followed by anything, meant as a test message or to specify that no actual message has been received yet)
  Heartbeat, // (not followed by anything, just meant to tell the other side that this side is still running)
  CloseConnection, // either side telling the other side to close the connection (not followed by anything)
  DesiredPaddleDisplacement, // followed by client sending delta x and delta z floats to move their paddle
  GameStatePositionsUpdate, // followed by server sending the two paddle and the puck positions to the client
  GameStatePuckStateUpdate, // followed by server sending the new puckstate int
  GameStateScoresUpdate, // followed by server sending two score ints
  GameStateMatchSecondsLeft // followed by server sending the number of seconds left in the current match (will be a negative number in the preMatch "Get Ready" phase)
}
