/* Project: Paddle Puck
 * Author: Dennis Busch, http://www.dennisbusch.de, Dennis[minus]Busch[at]gmx[dot]net
 */

package util;

import java.awt.List;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * This class provides logging functions for logging to the standard output
 * stream and optionally to an awt.List component
 */
public class Logging
{
  public List windowLog = null;
  
  private DateFormat df = new SimpleDateFormat("yyyy_MM_dd HH:mm:ss:SSS");
  private int maxLogItemsInView = 100;
  
  /**
   * Constructs a new Logging object.
   * @param logList the awt.List item which will receive the logged messages (set to null to disable logging to GUI)
   */
  public Logging(List logList)
  {
    this.windowLog = logList;
  }
  
  /**
   * Appends the message and stack-trace of the given exception to the log(s).
   * @param e 
   */
  public synchronized void LogException(Exception e)
  {
    if(e != null)
    {
      StringBuilder sb = new StringBuilder();
      for(int i=0; i<e.getStackTrace().length; i++)
      {
        sb.append(e.getStackTrace()[i].toString());
        if(i != e.getStackTrace().length - 1)
          sb.append(" / ");
      }

      this.LogMessage("Exception: " + e.getMessage() + " | Stacktrace: " + sb.toString());
    }
  }
  
  /**
   * Appends a given message to a datetime string and writes it to the log(s).
   * @param message the string to write to the log
   */
  public synchronized void LogMessage(String message)
  {
    Date dt = new Date();
    String formatted = String.format("%1$s > %2$s", df.format(dt), message);
    
    // add to console log
    System.out.println(formatted);
    
    // add to window log
    if(windowLog != null)
    {
      windowLog.add(formatted, 0); // insert newest message at the top
      
      if(windowLog.getItemCount() > maxLogItemsInView )
      {
        while(windowLog.getItemCount() > maxLogItemsInView)
        {
          windowLog.remove(windowLog.getItemCount()-1);
        }
      }
    }
    
    dt = null;
  }
}
