/* Project: Paddle Puck
 * Author: Dennis Busch, http://www.dennisbusch.de, Dennis[minus]Busch[at]gmx[dot]net
 */

package util.MyMath;

/**
 * a point in a 2D cartesian coordinate system
 */
public class Point2D
{
  public float x;
  public float y;
  
  public Point2D()
  {
    x = 0f;
    y = 0f;
  }
  
  public Point2D(float x, float y)
  {
    this.x = x;
    this.y = y;
  }
  
  /**
   * @return the length of the point interpreted as a 2D vector starting at 0,0
   */
  public float length()
  {
    return (float)Math.sqrt(x*x + y*y);
  }
  
  /**
   * multiplies each component with the given scale s
   * @param s 
   */
  public void scale(float s)
  {
    this.x *= s;
    this.y *= s;
  }
  
  /**
   * creates a 2D point from a 3D point, mapping x,z of the 3D point to x,y of the new 2D point
   * @param pos3D x,y,z of a 3D point
   * @return a 2D point holding x,z of the given 3D point as x,y
   */
  public static Point2D from3DmapXZtoXY(float[] pos3D)
  {
    if (pos3D.length == 3)
      return new Point2D(pos3D[0], pos3D[2]);
    else
      return new Point2D();
  }
  
  /**
   * creates a 2D point from a 3D point, mapping x,z of the 3D point to y,x of the new 2D point
   * @param pos3D x,y,z of a 3D point
   * @return a 2D point holding x,z of the given 3D point as y,x
   */
  public static Point2D from3DmapXZtoYX(float[] pos3D)
  {
    if (pos3D.length == 3)
      return new Point2D(pos3D[2], pos3D[0]);
    else
      return new Point2D();
  }
  
  /**
   * creates a 3D point from a 2D point, mapping x,y of the 2D point to x,z of the new 3D point
   * (this is the reverse method for from3DmapXZtoXY)
   * @param pos2D a 2D point
   * @return pos2D.x, 0, pos2D.z
   */
  public static float[] to3DmapXYtoXZ(Point2D pos2D)
  {
    return new float[] { pos2D.x, 0f, pos2D.y };
  }
  
  /**
   * creates a 3D point from a 2D point, mapping x,y of the 2D point to x,z of the new 3D point
   * and setting the y of the 3D point to the given y
   * @param pos2D a 2D point
   * @param float y
   * @return pos2D.x, y, pos2D.z
   */
  public static float[] to3DmapXYtoXZ(Point2D pos2D, float y)
  {
    return new float[] { pos2D.x, y, pos2D.y };
  }
}
