/* Project: Paddle Puck
 * Author: Dennis Busch, http://www.dennisbusch.de, Dennis[minus]Busch[at]gmx[dot]net
 */

package util;

import java.lang.reflect.Method;

/**
 * This class implements Runnable and inside the Run method
 * tries to invoke a parameterless method whose name along with the object
 * on which to invoke it are passed in with the constructor.
 */
public final class SimpleRunnable implements Runnable
{
  private Object owner;
  private Method method;
  
  /**
   * Creates and returns a new Thread object (using an instance of this class as the Runnable)
   * @param threadOwner must refer to an object whose class has an instance method called by the value given to the next parameter
   * @param threadRunMethodName must be the name of an instance method on threadOwners class (will be called in the run method of SimpleRunnable)
   * @param threadName the name for the created thread
   * @return a thread object using a SimpleRunnable
   * @throws Exception as thrown by the SimpleRunnable constructor
   */
  public static Thread CreateThread(Object threadOwner, String threadRunMethodName, String threadName) throws Exception
  {
    return new Thread(new SimpleRunnable(threadOwner, threadRunMethodName), threadName);
  }
  
  /**
   * Constructs a new SimpleRunnable object.
   * @param threadOwner must refer to an object whose class has an instance method called by the value given to the next parameter
   * @param threadRunMethodName must be the name of an instance method on threadOwners class (will be called in the run method of SimpleRunnable)
   * @throws Exception as thrown by getClass/getMethod reflection methods
   */
  public SimpleRunnable(Object threadOwner, String threadRunMethodName) throws Exception
  {
    this.owner = threadOwner;
  
    Class<? extends Object> aClass = this.owner.getClass();
    method = aClass.getMethod(threadRunMethodName, (Class<?>[]) null);
  }
  
  /**
   * tries to run the method as given in the constructor on the object also given in the constructor
   */
  @Override
  public void run()
  {
    try
    {
      method.invoke(owner, (Object[]) null);
    }
    catch(Exception e)
    {
      // safe to do nothing here as relevant exceptions are already thrown in constructors
      // so method is correctly initialized in any case and this exception should never occur
    }
  }
}
